
package org.greenstone.server;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;

import org.apache.log4j.*;

import org.greenstone.server.BaseServer;
import org.greenstone.server.BaseProperty;

public class Server2 extends BaseServer 
{
    protected String libraryURL;

    public Server2(String gsdl2_home, String lang)
    {
	super(gsdl2_home, lang,
	      gsdl2_home+File.separator+"lib"+File.separator+"java"+File.separator+"build.properties");	

	Property = new Server2Property();

	String frame_title = dictionary.get("ServerControl.Frame_Title");
	server_control_ = new Server2Control(this,frame_title);

	/** Make command tagets for managing Web server */
	START_CMD     = "web-start";
	RESTART_CMD   = "web-restart";
	CONFIGURE_CMD = "configure-web";
	STOP_CMD      = "web-stop";

	autoStart();
    }

    // Prepare the log4j.properties for GS2
    protected void initLogger() {

	String libjavaFolder = gsdl_home+File.separator+"lib"+File.separator+"java"+File.separator;
	File propsFile = new File(libjavaFolder+"log4j.properties");

	// create it from the template file log4j.properties.in
	if(!propsFile.exists()) {
	    try {
		// need to set gsdl2.home property's value to be gsdl_home,
		// so that the location  of the log files gets resolved correctly
		
		// load the template log4j.properties.in file into logProps
		FileInputStream infile = new FileInputStream(new File(libjavaFolder+"log4j.properties.in"));
		if(infile != null) {
		    Properties logProps = new Properties();
		    logProps.load(infile);
		    infile.close();

		    // set gsdl3.home to gsdl_home
		    logProps.setProperty("gsdl2.home", gsdl_home);
		    
		    // write the customised properties out to a custom log4j.properties file
		    FileOutputStream outfile = new FileOutputStream(propsFile);
		    if(outfile != null) {
			logProps.store(outfile, "Customised log4j.properties file");
			outfile.close();
		    } else {
			System.err.println("Could not store properties file " + propsFile + " for Server2.");
		    }
		}
	    } catch(Exception e) {
		System.err.println("Exception occurred when custom-configuring the logger for Server2.\n" + e);
	    }
	}

	// now configure the logger with the custom log4j.properties file
	if(propsFile.exists()) {
	    PropertyConfigurator.configure(propsFile.getAbsolutePath());
	} else {
	    System.err.println("Could not create properties file " + propsFile + " for Server2.");
	}
    }
    

    protected int runTarget(String cmd)
    {
	RunMake runMake = new RunMake();
	runMake.setTargetCmd(cmd);
	runMake.run();
   	return runMake.getTargetState();
    }

    public String getBrowserURL() {
	return libraryURL;
    }

    // works out the library URL again
    public void reload() {
	// default values, to be replaced with what's in gsdlsite.cfg
	String host = "localhost";
	String port = "8080";
	String gwcgi;
	String httpprefix = "greenstone";
	String suffix = "/cgi-bin/library.cgi";

	// get the prefix from the gsdlsite.cfg and build.properties files
	// (port number and servername)
	try{
	    File gsdlsite_cfg = new File(gsdl_home + File.separator + "cgi-bin" + File.separator + "gsdlsite.cfg");
	    FileInputStream fin = new FileInputStream(gsdlsite_cfg); 
	    Properties gsdlProperties = new Properties();
	    if(fin != null) {
		gsdlProperties.load(fin);
	    
		gwcgi = gsdlProperties.getProperty("gwcgi");
		if(gwcgi != null) {
		    suffix = gwcgi;
		} else {
		    httpprefix = gsdlProperties.getProperty("httpprefix", httpprefix);
		    suffix = httpprefix + suffix;
		}
		fin.close();
	    } else {
		recordError("Could not open gsdlsite_cfg for reading, using default library prefix.");
	    }
	    reloadBuildProperties();
	    host = build_properties.getProperty("apache.server", host);
	    port = build_properties.getProperty("apache.port", port);

	} catch(Exception e) {
	    recordError("Exception trying to load properties from gsdlsite_cfg. Using default library prefix.", e);
	    suffix = httpprefix + suffix;
	}

	libraryURL = "http://" + host + ":" + port + suffix;
	//recordSuccess("**** browser URL is: " + libraryURL);
	
    }
    

    public static void main (String[] args)
    {
    	if ((args.length < 1) || (args.length>2)) {
	    System.err.println("Usage: java org.greenstone.server.Server2 <gsdl2-home-dir> [lang]");
	    System.exit(1);
	}
	
	String gsdl2_home = args[0];
	File gsdl2_dir = new File(gsdl2_home);
	if (!gsdl2_dir.isDirectory()) {
	    System.err.println("gsdl-home-dir directory does not exist!");
	    System.exit(1);
	}
	
	String lang = (args.length==2) ? args[1] : "en";
	new Server2(gsdl2_home,lang);
    }
}
